'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var errUnit = function errUnit(errMsg) {
  return new Error('\x1B[33m' + errMsg + '\x1B[0m');
};

var closeReturnError = function closeReturnError(fd, errMsg, cb) {
  return _fs2.default.close(fd, function () {
    return cb(errUnit(errMsg));
  });
};

var cbMsg = 'Invalid callback function.';
var arrOfRegMsg = 'Invalid array type';
var arrOfRegEmptyMsg = 'Empty array of objects';
var regMsg = 'Invalid regex at: ';

var regLen = function regLen(regex) {
  return regex.length;
};

var isStr = function isStr(str) {
  return typeof str === 'string';
};

var isReg = function isReg(reg) {
  return (typeof reg === 'undefined' ? 'undefined' : _typeof(reg)) === 'object';
};

var find = function find(opt, cb) {
  var opts = opt || {};

  if (typeof cb !== 'function') throw errUnit(cbMsg);

  return _fs2.default.open(opts.path, 'r', function (openErr, fd) {
    if (openErr) return cb(openErr);

    if (!(opts.request instanceof Array)) return closeReturnError(fd, arrOfRegMsg, cb);

    var arrayLen = regLen(opts.request);
    if (arrayLen === 0) return closeReturnError(fd, arrOfRegEmptyMsg, cb);

    for (var i = 0; i < arrayLen; i += 1) {
      if (!isReg(opts.request[i]) && !isStr(opts.request[i])) {
        return closeReturnError(fd, regMsg + opts.request[i], cb);
      }
    }

    var isFlags = [];

    for (var _i = 0; _i < arrayLen; _i += 1) {
      isFlags[_i] = null;
    }
    var isFoundAll = false;
    var readStream = _fs2.default.createReadStream(opts.path, {
      encoding: opts.encoding || 'utf8'
    });
    readStream.on('error', function (readErr) {
      return closeReturnError(fd, readErr, cb);
    });
    var chunkQueue = [];
    var combinedChunk = void 0;
    var maxJoins = opts.join || 2;
    readStream.on('data', function (chunk) {
      if (chunkQueue.length < maxJoins) chunkQueue.push(chunk);else {
        chunkQueue.shift();

        chunkQueue.push(chunk);
      }

      combinedChunk = null;

      for (var j = 0; j < chunkQueue.length; j += 1) {
        combinedChunk += chunkQueue[j];
      }
      for (var _i2 = 0; _i2 < arrayLen; _i2 += 1) {
        if (isFlags[_i2] === null) {
          isFlags[_i2] = chunk.match(opts.request[_i2]) || combinedChunk.match(opts.request[_i2]);
        }
      }
      if (!isFlags.includes(null)) isFoundAll = true;
      if (isFoundAll) {
        combinedChunk = null;
        readStream.destroy();
        readStream.emit('end');
      }
    });
    return readStream.on('end', function () {
      return _fs2.default.close(fd, function () {
        var report = [];
        for (var _i3 = 0; _i3 < arrayLen; _i3 += 1) {
          report[_i3] = {
            isFound: isFlags[_i3] !== null,
            reg: opts.request[_i3],
            match: isFlags[_i3]
          };
        }
        return cb(null, report);
      });
    });
  });
};

exports.default = find;